part of masamune_agora;

/// Bucket settings for cloud recording.
class AgoraStorageBucketConfig extends StorageBucketConfig {
  /// Bucket settings for cloud recording.
  ///
  /// The [bucketName] is not the Amazon resource name, but the name as it is given.
  const AgoraStorageBucketConfig({
    required this.accessKey,
    this.region = AWSRegion.ap_northeast_1,
    required this.bucketName,
    required this.secretKey,
    this.path = "",
  });

  /// Access key.
  final String accessKey;

  /// Region.
  final AWSRegion region;

  /// Bucket name.
  ///
  /// The [bucketName] is not the Amazon resource name, but the name as it is given.
  final String bucketName;

  /// Secret key.
  final String secretKey;

  /// Pass to save.
  final String path;

  static AgoraStorageBucketConfig? _fromMap(DynamicMap map) {
    if (!map.containsKey("accessKey") ||
        !map.containsKey("bucketName") ||
        !map.containsKey("secretKey")) {
      return null;
    }

    return AgoraStorageBucketConfig(
      accessKey: map.get("accessKey", ""),
      bucketName: map.get("bucketName", ""),
      secretKey: map.get("secretKey", ""),
      region: AWSRegion.values.firstWhereOrNull((e) =>
              e.index == map.get("region", AWSRegion.ap_northeast_1.index)) ??
          AWSRegion.ap_northeast_1,
      path: map.get("path", ""),
    );
  }

  @override
  DynamicMap toMap() {
    return <String, dynamic>{
      "accessKey": accessKey,
      "region": region.index,
      "bucketName": bucketName,
      "secretKey": secretKey,
      "path": path,
    };
  }
}

extension _AgoraStorageBucketConfigDynamicMapExtensions on DynamicMap? {
  AgoraStorageBucketConfig? _toAgoraStorageBucketConfig() {
    if (this == null) {
      return null;
    }
    return AgoraStorageBucketConfig._fromMap(this!);
  }
}

/// AWS Regions.
enum AWSRegion {
  us_east_1,
  us_east_2,
  us_west_1,
  us_west_2,

  eu_west_1,
  eu_west_2,
  eu_west_3,
  eu_central_1,

  ap_southeast_1,
  ap_southeast_2,
  ap_northeast_1,
  ap_northeast_2,

  sa_east_1,

  ca_central_1,

  ap_south_1,

  cn_north_1,
  cn_northwest_1,
  us_gov_west_1,
}
